package tctbridge.io;

// Infomat
import infomat.vectorspace.*;

// TCT
import text.data.Corpus;
import core.clustering.*;
import core.clustering.util.*;
import core.metrics.similarity.*;
import matrix.dense.DenseVector;

// Java
import java.io.*;
import java.util.*; 
import java.util.zip.*;
import java.awt.Dimension;


/**
 * For transfering matrix and related info to and from the TCT package.
 * Implemented to work when the rows are texts and the columns words.
 *
 * Note: does not work on is own! Has to be used through the 
 * infomat.grouping.tct-package.
 *
 * TCT - Text Clustering Toolkit, see http://mlg.ucd.ie/content/view/18/
 * 
 * @version          080305
 * @author           Magnus Rosell
 */
public class TCTBridge{
    
    private IndexMap objIndexMap = null;
    private IndexMap featureIndexMap = null;

    /**
     * Returns a TCT Corpus built from the IMatrix. 
     * The order of the IObject:s in the IMatrix is essential here.
     * To get a clustering result back using #getIObjectGrouping()
     * the order must not be changed.
     */    
    public Corpus getCorpus(IMatrix matrix, IObjectGroup group){
	objIndexMap = new IndexMap();
	featureIndexMap = new IndexMap();

	IObjectGroup featureGroup = matrix.constructCorrespondingGroup(group);

	IObjectGroup rowGroup = featureGroup;
	IObjectGroup colGroup = group;
	if(matrix.rowObject(group.getOrdered().elementAt(0))){
	    rowGroup = group;
	    colGroup = featureGroup;
	}
	if( (rowGroup.numberOfIObjects() 
	     < matrix.getRowIObjectSet().numberOfIObjects()) ||
	    (colGroup.numberOfIObjects() 
	     < matrix.getColumnIObjectSet().numberOfIObjects()) )
	    matrix = matrix.subMatrix(group, featureGroup);

	/*
	IObjectSet featureSet = matrix.getColumnIObjectSet();
	if(featureSet==group)
	    featureSet = matrix.getRowIObjectSet();
	*/

	int docCount = group.numberOfIObjects();
	int termCount = featureGroup.numberOfIObjects();

	System.out.println("TCTBridge. Matrix: "+matrix.getSize());

	System.out.println("doc, term: "+docCount+", "+termCount);

	Corpus corpus = new Corpus(docCount, termCount);

	Vector cells = matrix.getAllCells();
	for(int i=0; i<cells.size(); i++){
	    IMatrixCell mc = (IMatrixCell) cells.elementAt(i);
	    int objIndex = group.getIObjectOrderIndex(mc.getRowIObject());
	    int tctObjIndex = objIndexMap.add(objIndex);

	    int featureIndex = 
		featureGroup.getIObjectOrderIndex(mc.getColumnIObject());
	    int tctFeatureIndex = featureIndexMap.add(featureIndex);

	    double value = mc.getWeight();
	    //	    corpus.setData(objIndex, featureIndex, value);
	    corpus.setData(tctObjIndex, tctFeatureIndex, value);
	}

	/*
	Vector<IObject> vec = group.getOrdered();
	for(int i=0; i<vec.size(); i++){
	    System.out.println(i+": "+vec.elementAt(i).getString());
	}
	*/

	return corpus;
    }

    private class IndexMap{
	private int counter = 0;

	private HashMap<Integer, Integer> toMap = 
	    new HashMap<Integer, Integer>();
	
	private HashMap<Integer, Integer> fromMap = 
	    new HashMap<Integer, Integer>();
	
	public int add(int index){
	    Integer count = toMap.get(index);
	    if(count==null){
		toMap.put(index, counter);
		fromMap.put(counter, index);
		count = counter;
		counter++;
	    }
	    return count;
	}
	
	public int getOriginal(int tctIndex){
	    return fromMap.get(tctIndex);
	}
    }


    /**
     * Returns a IObjectGrouping constructed from the TCT HardClustering.
     * Used together with #getCorpus().
     * The IMatrix and Corpus must be the same.
     * To get the objects in order within the clusters a metric has to be
     * given, use the same as in the clustering method you have used.
     * The constructor object could be anything you would like the
     * IObjectGrouping to get its name from (uses the class name of 
     * that object).
     * @param simOrder wether the objects are sorted in order of similarity
     * to their cluster centroid (within each cluster)
     */
    public IObjectGrouping getIObjectGrouping(IMatrix matrix,
					      IObjectGroup group, 
					      Corpus corpus,
					      HardClustering clustering,
					      SimilarityMetric metric,
					      boolean simOrder, 
					      Object constructor){
	
	String name = "TCT "+constructor.getClass().getName();
	IObjectGrouping grouping = new IObjectGrouping(name);

	DenseVector[] centroids = null;
	if(simOrder)
	    centroids = ClusterUtils.buildCentroids(clustering);

	ArrayList<Integer>[] clusterLists = ClusterUtils.toLists(clustering);
	for(int i=0; i<clusterLists.length; i++){
	    IObjectGroup cluster = new IObjectGroup("TCT Group " +i);
	    ArrayList<Integer> al = clusterLists[i];
	    for(int j=0; j<al.size(); j++){
		/*
		int objIndex = al.get(j);
		IObject io = group.getIObjectAt(objIndex);
		*/

		int tctObjIndex = al.get(j);
		int objIndex = objIndexMap.getOriginal(tctObjIndex);
		IObject io = group.getIObjectAt(objIndex);

		float orderValue = al.size()-j;
		if(simOrder)
		    orderValue = (float) metric.similarity(centroids[i], corpus.getObject(objIndex));
		cluster.add(io, orderValue);
	    }
	    cluster.sortInOrderValueOrder();
	    grouping.add(cluster);
	}

	grouping.sort();
	//	grouping.printGroupsObjects(100);

	return grouping;
    }
	


    /*
    public IMatrix getMatrix(Corpus corpus){
	IMatrix matrix = new IMatrix();
    }
    */

    /*
    public void saveIMatrix(IMatrix matrix, File file) throws IOException{
	PrintWriter fw = new PrintWriter(new FileWriter(file));

	Dimension d = matrix.getSize();
	fw.println(d.getHeight()+" "+d.getWidth());
	

	IObjectSet rows = matrix.getRowIObjectSet();
	Vector<IObject> rowVec = rows.getOrdered();
	HashMap<IObject, Integer> rowNrMap = new HashMap<IObject, Integer>();
	for(int i=0; i<rowVec.size(); i++)
	    rowNrMap.put(rowVec.elementAt(i), i);
	IObjectTCTComparator iotctc = new IObjectTCTComparator(rowNrMap);

	IObjectSet cols = matrix.getColumnIObjectSet();
	Vector<IObject> colVec = cols.getOrdered();
	HashMap<IObject, Integer> colNrMap = new HashMap<IObject, Integer>();
	for(int i=0; i<colVec.size(); i++){
	    IObject colO = colVec.elementAt(i);
	    colNrMap.put(colO, i);
	    HashSet cellSet = matrix.getRowIMatrixCellsForColumnObject(colO);
	    Vector cells = new Vector(cellSet);
	    Collections.sort(cells, iotctc);
	    for(int j=0; j<cells.size(); j++){
		IMatrixCell mc = (IMatrixCell) cells.elementAt(j);
		fw.println(((int) i)+" "+((int) j)+" "+
			   ((int) mc.getLocalCount()));
	    }
	}
    }
    */



}

/*
class IObjectTCTComparator implements Comparator{
    
    HashMap<IObject, Integer> objNrMap = null;

    public IObjectTCTComparator(HashMap<IObject, Integer> objNrMap){
	this.objNrMap=objNrMap;
    }

    public int compare(Object o1, Object o2) {
	IMatrixCell mc1 = (IMatrixCell) o1;
	IMatrixCell mc2 = (IMatrixCell) o2;
	Integer i1 = objNrMap.get(mc1.getRowIObject());
	Integer i2 = objNrMap.get(mc2.getRowIObject());
	return i1.compareTo(i2);
    }

}
*/
