package infomat.vectorspace.vector;

import infomat.io.*;
import mro.util.*;
import mro.util.doublehashmap.DoubleHashMap;

import java.io.*;
import java.util.*;
import java.awt.Dimension;

import javax.swing.JLabel;


/**
 * An implementation of a sparse vector. Should be thought of as a 
 * java.util.HashMap rather than a java.util.Vector.
 * Contains Objects that implements
 * the IVectorIndex interface. Consisits of a hashmap,
 * mapping such objects to Float:s.
 *
 * @version          080623
 * @author           Magnus Rosell
 */
public class IVector<T extends IVectorIndex>{

    /**
     * The sparse representation, a map from T:s to Float values.
     */
    private HashMap<T, Float> objectValueMap = 
	new HashMap<T, Float>();

    public IVector(){}


    /**
     * Copies the map.
     */
    public IVector(HashMap<T, Float> map){
	objectValueMap = new HashMap<T, Float>(map);
    }

    /**
     * Copies the map.
     */
    public IVector(IVector<T> vec){
	objectValueMap = new HashMap<T, Float>(vec.objectValueMap);
    }

    /**
     * Adds the value f to the previosus value for the T t.
     * If this mapping does not exist it is created with the value f.
     */
    public void add(T t, Float f){
	Float f2 = objectValueMap.get(t);
	if(f2!=null)
	    f += f2;
	objectValueMap.put(t, f);
    }
    
    /**
     * Uses the add(T, Float) method.
     */ 
    public void add(IVector<T> vec){
	add(vec.objectValueMap);
    }

    /**
     * Uses the add(T, Float) method.
     */ 
    public void add(HashMap<T, Float> map){
	Iterator<Map.Entry<T, Float>> it = map.entrySet().iterator();
	while(it.hasNext()){
	    Map.Entry<T, Float> entry = it.next();
	    add(entry.getKey(), entry.getValue());
	}
    }

    /**
     * Sets the value for t to be f.
     */
    public void put(T t, Float f){
	objectValueMap.put(t, f);
    }

    /**
     * Uses the put(T, Float) method.
     */ 
    public void put(IVector<T> vec){
	put(vec.objectValueMap);
    }

    /**
     * Uses the add(T, Float) method.
     */ 
    public void put(HashMap<T, Float> map){
	Iterator<Map.Entry<T, Float>> it = map.entrySet().iterator();
	while(it.hasNext()){
	    Map.Entry<T, Float> entry = it.next();
	    put(entry.getKey(), entry.getValue());
	}
    }



    public void remove(T t){
	objectValueMap.remove(t);
    }

    public void removeAll(){
	objectValueMap = 
	    new HashMap<T, Float>();
    }


    /**
     * Subtracts this value from the value for the T.
     */ 
    public void subtract(T t, Float f){
	f = - f;
	Float f2 = objectValueMap.get(t);
	if(f2!=null)
	    f = f2 + f;
	objectValueMap.put(t, f);
    }

    /**
     * Uses the subtract(T, Float) method.
     */ 
    public void subtract(IVector<T> vec){
	subtract(vec.objectValueMap);
    }

    /**
     * Uses the subtract(T, Float) method.
     */ 
    public void subtract(HashMap<T, Float> map){
	Iterator<Map.Entry<T, Float>> it = map.entrySet().iterator();
	while(it.hasNext()){
	    Map.Entry<T, Float> entry = it.next();
	    subtract(entry.getKey(), entry.getValue());
	}
    }


    /**
     * Multiplies this value with the value for the T.
     */ 
    public void multiply(T t, Float f){
	Float f2 = objectValueMap.get(t);
	if(f2!=null){
	    f = f2 * f;
	    objectValueMap.put(t, f);
	}
    }

    /**
     * Uses the multiply(T, Float) method - 
     * multiplies all elements by the the provided value.
     */ 
    public void multiply(Float f){
	Iterator<Map.Entry<T, Float>> it = 
	    objectValueMap.entrySet().iterator();
	while(it.hasNext()){
	    Map.Entry<T, Float> entry = it.next();
	    multiply(entry.getKey(), f);
	}
    }

    /**
     * Sets this value to its logarithm with the specified base.
     */
    public void log(T t, double base){
	Float f = objectValueMap.get(t);
	if(f!=null){
	    f = (float) MoreMath.log(f, base);
	    objectValueMap.put(t, f);
	}
    }

    /**
     * Uses the log(T, double) method - 
     * takes the logarithm of all elements using the provided base.
     */ 
    public void log(double base){
	Iterator<Map.Entry<T, Float>> it = 
	    objectValueMap.entrySet().iterator();
	while(it.hasNext()){
	    Map.Entry<T, Float> entry = it.next();
	    log(entry.getKey(), base);
	}
    }

    public void normalize(){
	double sqsum = 0.0;
	Iterator<Map.Entry<T, Float>> it = 
	    objectValueMap.entrySet().iterator();
	while(it.hasNext()){
	    Map.Entry<T, Float> entry = it.next();
	    sqsum += Math.pow(entry.getValue(), 2);
	}
	it = objectValueMap.entrySet().iterator();
	while(it.hasNext()){
	    Map.Entry<T, Float> entry = it.next();
	    objectValueMap.put(entry.getKey(), (float) Math.sqrt(Math.pow(entry.getValue(), 2)/sqsum));
	}
    }


    /**
     * Returns the n highest ranked or as many as there is.
     */
    public IVector<T> highestRanked(int n){
	IVector<T> ret = new IVector<T>();
	Vector<T> ordered = getOrderedVector();	
	for(int i=0; i<ordered.size() && i<n; i++){
	    T t = ordered.elementAt(i);
	    Float f = getValue(t);
	    ret.add(t,f);
	}
	return ret;
    }

    /**
     * Returns only those that larger than the specified value.
     */
    public IVector<T> largerThan(float min){
	IVector<T> ret = new IVector<T>();
	Iterator<Map.Entry<T, Float>> it = 
	    objectValueMap.entrySet().iterator();
	while(it.hasNext()){
	    Map.Entry<T, Float> entry = it.next();
	    if(entry.getValue() > min)
		ret.add(entry.getKey(), entry.getValue());
	}
	return ret;
    }

    /*
    public IVector<T> diff(IVector<T> other){
	
    }

    public static Vector<T> 
	intersection(IVector<T> first, IVector<T> second){

	Vector<T> ret = new Vector<T>(); 
	IVector<T> small = first;
	IVector<T> large = second;
	if(small.size() > large.size()){
	    small=second;
	    large=first;
	}
	Iterator<Map.Entry<T, Float>> it = small.iterator();
	while(it.hasNext()){
	    Map.Entry<T, Float> entry = it.next();
	    Float f = large.getValue(it.getKey());
	    if(f!=null)
		ret.add(it.getKey());
	}
	return ret;
    }
    */

    /**
     * Returns the scalar product of the two IVector:s.
     *
     * <br><br>
     * Checks whether the second IVector is smaller
     * than first. A small decrease in time is achieved by 
     * giving the IVector that one believes is smaller 
     * as first parameter.
     * (Do not check it also. That increases time spent.)
     */
    public static <U extends IVectorIndex, V extends IVectorIndex> 
	float dotProduct(IVector<U> vec1, IVector<V> vec2){

	double prod = 0.0f;

	HashMap<U, Float> smallMapA = vec1.getMap();
	HashMap<V, Float> largeMapA = vec2.getMap();
	if(largeMapA.size()<smallMapA.size()){
	    HashMap<V, Float> smallMapB = vec2.getMap();
	    HashMap<U, Float> largeMapB = vec1.getMap();

	    Iterator<Map.Entry<V, Float>> it = 
		smallMapB.entrySet().iterator();
	    while(it.hasNext()){
		Map.Entry<V, Float> entry = it.next();
		Float largeProd = largeMapB.get(entry.getKey());
		if(largeProd!=null)
		    prod += largeProd * entry.getValue();
	    }
	}else{
	    Iterator<Map.Entry<U, Float>> it = 
		smallMapA.entrySet().iterator();
	    while(it.hasNext()){
		Map.Entry<U, Float> entry = it.next();
		Float largeProd = largeMapA.get(entry.getKey());
		if(largeProd!=null)
		    prod += largeProd * entry.getValue();
	    }
	}

	return (float) prod;
    }

    /*
    public static <T extends IVectorIndex> 
	float dotProduct(IVector vec1, IVector vec2){

	HashMap<T, Float> smallMap = vec1.getMap();
	HashMap<T, Float> largeMap = vec2.getMap();
	if(largeMap.size()<smallMap.size()){
	    HashMap<T, Float> tmpMap = smallMap;
	    smallMap = largeMap;
	    largeMap = smallMap;
	}

	double prod = 0.0f;
	Iterator<Map.Entry<T, Float>> it = 
	    smallMap.entrySet().iterator();
	while(it.hasNext()){
	    Map.Entry<T, Float> entry = it.next();
	    Float largeProd = largeMap.get(entry.getKey());
	    if(largeProd!=null)
		prod += largeProd * entry.getValue();
	}

	return (float) prod;
    }
    */

    /**
     * Returns the number of indexes used in this IVector.
     */
    public int size(){
	return objectValueMap.size();
    }

    /**
     * Returns the values for the specified T.
     */
    public Float getValue(T t){
	return objectValueMap.get(t);
    }

    public boolean contains(T t){
	if(getValue(t)==null)
	    return false;
	return true;
    }

    /** 
     * Returns the actual map. Not a copy!
     */
    public HashMap<T, Float> getMap(){
	return objectValueMap;
    }

    public String toString(){
	String ret = "";
	Iterator<Map.Entry<T, Float>> it = 
	    objectValueMap.entrySet().iterator();
	while(it.hasNext()){
	    Map.Entry<T, Float> entry = it.next();
	    ret += "("+entry.getKey()+", "+entry.getValue()+") ";
	}
	return ret;
    }

    
    /**
     * Returns a vector of T:s, orderd by their value.
     */
    public Vector<T> getOrderedVector(){
	Vector<T> ret = new Vector<T>(objectValueMap.keySet());
	Collections.sort(ret, new TValueComparator(objectValueMap));
	return ret;
    }

    /**
     * Returns a vector of T:s, in no particular order.
     * Faster than #getOrderedVector().
     */
    public Vector<T> getUnorderedVector(){
	Vector<T> ret = new Vector<T>(objectValueMap.keySet());
	return ret;
    }

    public void sortLiteral(){
	Vector<T> vec = new Vector<T>(objectValueMap.keySet());
	Collections.sort(vec, new TLiteralComparator());
	removeAll();
	for(int i=0; i<vec.size(); i++){
	    add(vec.get(i), (float) vec.size()-i-1);
	}
    }

    public void sortRandom(){
	Vector<T> vec = new Vector<T>(objectValueMap.keySet());
	Collections.shuffle(vec);
	removeAll();
	for(int i=0; i<vec.size(); i++){
	    add(vec.get(i), (float) vec.size()-i-1);
	}
    }

    public void sortInvert(){
	Vector<T> vec = getOrderedVector();
	if(vec.size()==0)
	    return;
	HashMap<T, Float> tmpObjectValueMap = getMap();
	float max = getValue(vec.get(0));
	removeAll();
	for(int i=0; i<vec.size(); i++){
	    T t = vec.get(i);
	    add(t, -(tmpObjectValueMap.get(t)-max));
	}
    }


    /**
     * For debugging.
     */
    public void print(){
	Vector<T> vec = getOrderedVector();
	for(int i=0; i<vec.size(); i++){
	    T t = vec.get(i);
	    System.out.println(t.getString()+": "+getValue(t));
	}
    }

    /*
    public void sortInvert(){
	Vector<T> vec = getOrdered();
	if(vec.size()==0)
	    return;
	HashMap<T, Float> tmpObjectValueMap = getMap();
	removeAll();
	for(int i=0; i<vec.size(); i++){
	    T t = vec.get(i);
	    add(t, tmpObjectValueMap.get(vec.get(vec.size()-i-1)));
	}
    }
    */

    /*
    public Vector<Map.Entry<T, Float>> getOrderedMapEntryVector(){
	return new Vector<Map.Entry<T, Float>>(objectValueMap.entrySet());
    }
    */

    public Iterator<Map.Entry<T, Float>> iterator(){
	return objectValueMap.entrySet().iterator();
    }

    /**
     * Private class for ordering the Ts in value order.
     */
    private class TValueComparator implements Comparator<T>{
	private HashMap<T, Float> objectValueMap;
	public TValueComparator(HashMap<T, Float> objectValueMap){
	    this.objectValueMap=objectValueMap;
	}

	public int compare(T t1, T t2){
	    float f1 = objectValueMap.get(t1);
	    float f2 = objectValueMap.get(t2);
	    return Float.compare(f2, f1);
	}
    }

    /**
     * Private class for ordering the Ts in literal order.
     * Uses the IVectorIndex.getString() - method.
     */
    private class TLiteralComparator implements Comparator<T>{

	public int compare(T t1, T t2){
	    String s1 = t1.getString();
	    String s2 = t2.getString();
	    return s1.compareTo(s2);
	}
    }


}
