package mro.util.experimentation;

import mro.util.*;
import mro.util.doublehashmap.*;

import java.io.*;
import java.util.*; 

import org.xml.sax.Attributes;
import org.xml.sax.InputSource;
import org.xml.sax.XMLReader;
import org.xml.sax.helpers.DefaultHandler;
import org.xml.sax.helpers.XMLReaderFactory;
import org.xml.sax.SAXException;


/**
 * A class for handling xml tables.
 *
 * @version          080429
 * @author           Magnus Rosell
 */

public class Table<V>{

    private GenericOrderedDoubleHashMap<Integer, Integer, V> map = 
	new GenericOrderedDoubleHashMap<Integer, Integer, V>();

    public void put(int r, int c, V v){
	map.put(r, c, v);
    }

    public GenericOrderedDoubleHashMap<Integer, Integer, V> 
	getGenericOrderedDoubleHashMap(){

	return new GenericOrderedDoubleHashMap<Integer, Integer, V>(map);
    }

    public void print(){
	map.print();
    }


    /********* IO *********/

    /**
     * Saves a Table-object to a file with the specified file name.
     */
    public void save(String fileName) throws IOException{
	save(new File(fileName));
    }

    /**
     * Saves a Table-object to the specified file.
     */
    public void save(File file) throws IOException{
	//	try {
	    File dir = file.getParentFile();
	    if(!dir.exists())
		dir.mkdirs();

	    // XSL
	    String xslFileName = file.getParent()+File.separator+
		"Table.xsl";

	    PrintWriter out = new PrintWriter(new BufferedWriter(new BufferedWriter(new FileWriter(xslFileName))));
	    printXSL(out);
	    out.close();

	    // XML
	    out = new PrintWriter(new BufferedWriter(new BufferedWriter(new FileWriter(file))));

	    //            printXMLHead(out);
	    printXML(out);
	    out.close();
	    /*
	}
	catch(IOException ioE) {
	    System.out.println(ioE);
	    ioE.printStackTrace();
	}
	    */
    }

    /**
     * Prints the complete DTD of a Table-XML-file to a 
     * {@link java.io.PrintWriter}.
     */
    public static void printDTD(PrintWriter pw) throws IOException{
	pw.println("<!DOCTYPE table [");
	printPartDTD(pw);

	pw.println("]>");
    }

    /**
     * Prints the DTD of a Table-XML-file to a 
     * {@link java.io.PrintWriter}. Only the elements.
     * This method may be used when the Table-XML should
     * be part of a larger XML.
     */
    public static void printPartDTD(PrintWriter pw) throws IOException{
	pw.println("<!--A table-->");
	pw.println("<!ELEMENT table (r*)>");

	pw.println("<!--A row-->");
	pw.println("<!ELEMENT r (c*)>");

	pw.println("<!--A column-->");
	pw.println("<!ELEMENT c (#PCDATA)>");
    }

    /**
     * Prints a reference to the Table-XSL-file to a 
     * {@link java.io.PrintWriter}.
     */
    public static void printTableXSLReference(PrintWriter pw) throws IOException{
	pw.println("<?xml-stylesheet type=\"text/xsl\" href=\"Table.xsl\"?>");
    }

    /**
     * Prints the Table to a 
     * {@link java.io.PrintWriter}.
     */
    public void printXML(PrintWriter pw) throws IOException{
	XMLHelper.printXMLHead(pw);
	printTableXSLReference(pw); 
	printDTD(pw);
	printPartXML(pw);
    }

    public void printPartXML(PrintWriter pw) throws IOException{
	pw.println("<table>");
	Vector<Integer> vec1 = map.getFirstIndexOrderVector();
	Vector<Integer> vec2 = map.getSecondIndexOrderVector();
	for(int i=0; i<vec1.size(); i++){
	    pw.print("<r>");
	    for(int j=0; j<vec2.size(); j++){
		pw.print("<c>");
		pw.print(map.get(vec1.elementAt(i), vec2.elementAt(j)));
		pw.print("</c>");
	    }
	    pw.println("</r>");
	}
	pw.println("</table>");
    }


    /**
     * Prints a basic XSL-file for Table XML-files to a 
     * {@link java.io.PrintWriter}.
     */
    public void printXSL(PrintWriter pw) throws IOException{
	pw.println("<?xml version=\"1.0\" encoding=\"ISO-8859-1\"?>");
	pw.println("<xsl:stylesheet version=\"1.0\"");
	pw.println("xmlns:xsl=\"http://www.w3.org/1999/XSL/Transform\">");
	pw.println("<xsl:template match=\"/\">");
	pw.println("<html>");
	pw.println("<body>");
	pw.println("<table border=\"1\">");

	pw.println("<xsl:apply-templates/>");

	pw.println("</table>");
	pw.println("</body>");
	pw.println("</html>");
	pw.println("</xsl:template>");


	pw.println("<xsl:template match=\"r\">");
	pw.println("<tr>");
	pw.println("<xsl:for-each select=\"c\">");
	pw.print("<td><xsl:value-of select=\".\"/></td>");
	pw.println("\n</xsl:for-each>");
	pw.println("</tr>");
	pw.println("</xsl:template>");

	pw.println("</xsl:stylesheet>");

    }


    /**
     * Loads a Table-object from a file with the specified file name.
     */
    public static Table load(String fileName)
	throws IOException, SAXException{
	return load(new File(fileName));
    }


    /**
     * Loads a Table-object from the specified file.
     */
    public static Table load(File file)
	throws IOException, SAXException{
	Table table = null;
	BufferedReader in = new BufferedReader(new FileReader(file));
	InputSource source = new InputSource(in);

	    //	    Table iop = new Table("Dummy");
	table=loadXML(source);
	    /*
	catch(IOException ioE) {
	    System.out.println(ioE);
	    ioE.printStackTrace();
	}
	    */
	return table;
    }



    /**
     * Loads a Table-object from an {@link java.io.InputSource}.
     */
    public static Table loadXML(InputSource source) 
	throws IOException, SAXException{
	Table table = null;
	//	try {
	    // Create parser+contentHandler and read the XML document
	    XMLReader parser = XMLReaderFactory.createXMLReader();
	    TableHandler handler = new TableHandler();
	    parser.setContentHandler(handler);
	    parser.parse(source);
	    table = handler.getTable();
	    /*
	} catch (Exception e) {
	    System.err.println(e);
	    e.printStackTrace();
	}
	    */
	return table;
    }

}

/**
 * Class used for XML-parsing of {@link Table}-objects.
 */
class TableHandler extends DefaultHandler {

    private StringBuffer buffer;

    private Table<String> table = new Table<String>();

    private int row = -1;
    private int col = -1;

    public Table getTable(){
	return table;
    }


    public void startElement (String namespaceURI, String localName,
			      String qualifiedName, Attributes atts) {
	buffer = new StringBuffer();

	if(qualifiedName.equals("r")){          
	    row++;
	    col=-1;
	}else if(qualifiedName.equals("c"))
	    col++;
    }


    public void endElement (String namespaceURI, String localName, String qualifiedName) {

	if(qualifiedName.equals("c")){
	    /*
	    String s = buffer.toString();
	    System.out.println(row +", "+col+": "+s);
	    table.put(row, col, s);
	    */
	    table.put(row, col, buffer.toString());
	}
    }

    public void characters (char ch[], int start, int length) {
	if(buffer != null)
	    buffer.append(ch,start,length);
    }      
}

