package mro.util.properties;

import mro.util.*;

import java.io.*;
import java.util.*; 

import org.xml.sax.Attributes;
import org.xml.sax.InputSource;
import org.xml.sax.XMLReader;
import org.xml.sax.helpers.DefaultHandler;
import org.xml.sax.helpers.XMLReaderFactory;

/**
 * A class that handles several Properties.
 * The Properties are stored mapped to the owning objects class name.
 * Note: Only handles one Properties per class name, meaning that
 * you can not have several instances of the same class.
 *
 * You can receive all Properties for a super class, 
 * that is a Properties for each sub class.
 *
 * @see Properties
 *
 * @version          080325
 * @author           Magnus Rosell
 */

public class PropertiesPool{

    private HashMap<String, Properties> classNamePropertiesMap = 
	new HashMap<String, Properties>();

    private XMLFileFilter xmlFF = new XMLFileFilter();


    public PropertiesPool(){}

    public PropertiesPool(PropertiesPool pool){
	Iterator<String> it = pool.classNamePropertiesMap.keySet().iterator();
	while(it.hasNext()){
	    String s = it.next();
	    try{
		Properties ps = 
		    new Properties(pool.classNamePropertiesMap.get(s));
		classNamePropertiesMap.put(s, ps);
	    }catch(PropertyNameConflictException pnce){
		// Won't happen here.
	    }
	}
    }


    /**
     * Loads all Properties stored as files in a directory.
     */
    public void loadPropertiesInDirectory(String dirName) 
	throws IOException, org.xml.sax.SAXException{
	loadPropertiesInDirectory(new File(dirName));
    }

    /**
     * Loads all Properties stored as files in a directory.
     * Reads only files with the ".xml"-extension.
     */
    public void loadPropertiesInDirectory(File dir) 
	throws IOException, org.xml.sax.SAXException{
	if(!dir.isDirectory()){
	    System.out.println(dir.getPath());
	    throw new IOException("PropertiesPool: Not a directory!");
	}
	File[] list = dir.listFiles(xmlFF);
	for(int i=0; i<list.length; i++){
	    if(!list[i].isDirectory()){
		//		System.out.println(" - "+list[i].getName());
		Properties p = Properties.load(list[i]);
		if(p!=null)
		    classNamePropertiesMap.put(p.getOwnerClassName(), p);
	    }
	}
    }


    /**
     * Saves all Properties as files in a directory.
     */
    public void savePropertiesToDirectory(String dirName) throws IOException{
	savePropertiesToDirectory(new File(dirName));
    }

    /**
     * Saves all Properties as files in a directory.
     */
    public void savePropertiesToDirectory(File dir) throws IOException{
	if(!dir.isDirectory())
	    throw new IOException("PropertiesPool: Not a directory!");
	
	String name = dir.getPath();
	if(!name.endsWith(File.separator))
	    name+=File.separator;

	long time = System.currentTimeMillis();
	Iterator<Properties> it = classNamePropertiesMap.values().iterator();
	while(it.hasNext()){
	    Properties props = it.next();
	    props.save(name+props.getPropertiesName()+"."+time+".xml");
	}
    }

    /**
     * Returns a Vector of Properties, with those Properties
     * that store info on classes with the (by name) given super class.
     * Returns null if the superclass does not exist.
     */
    public Vector<Properties> 
	getPropertiesWithSuperClassName(String superClassName){
	Vector<Properties> propVec = new Vector<Properties>();

	Class superClass = null;
	try{
	    superClass = Class.forName(superClassName);
	}catch(ClassNotFoundException cnfe){
	    return null;
	}	    

	Iterator<String> it = classNamePropertiesMap.keySet().iterator();
	while(it.hasNext()){
	    String thisClassName = it.next();
	    try{
		Class thisClass = Class.forName(thisClassName);
		Class thisSuperClass = thisClass.getSuperclass();

		if(superClass == thisSuperClass)
		    propVec.add(classNamePropertiesMap.get(thisClassName));
	    }catch(ClassNotFoundException cnfe){
		// Do nothing
	    }	    
	}
	return propVec;
    }

    /**
     * Returns the Properties for the class with the given name,
     * null if there is not any such Properties.
     */
    public Properties getPropertiesWithClassName(String className){
	return classNamePropertiesMap.get(className);
    }


    /**
     * Returns the Properties stored for the class of the specified object.
     */
    public Properties getPropertiesForObject(Object o){
	return classNamePropertiesMap.get(o.getClass().getName());
    }

    /**
     * Returns the number of Properties in this PropertiesPool.
     */
    public int size(){
	return classNamePropertiesMap.size();
    }

    /**
     * For debugging purposes.
     */
    public void printClassNames(){
	System.out.println("Class Names:");
	Iterator<String> it = classNamePropertiesMap.keySet().iterator();
	while(it.hasNext())
	    System.out.println(" - "+it.next());
    }

}


class XMLFileFilter implements FileFilter{

    public boolean accept(File pathname){
	if(pathname.getName().toLowerCase().endsWith(".xml"))
	    return true;
	return false;
    }
}
